#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################


# --------------------------------------------------------------------------
# DESCRIPTION
# Initilizator of the program
# --------------------------------------------------------------------------


# GENERAL CONSTANTS
# -----------------------------
encoding system {utf-8}					;# System encoding
set LIB_DIRNAME	[file normalize [file dirname $argv0]]	;# Path to directory where the *.tcl file are located
set VERSION	"1.1"					;# Program version
set SHORTNAME	"MCU8051IDE"				;# Program short name (without white space)
set APPNAME	"MCU 8051 IDE v$VERSION"		;# Full program name
set APPLICATION_LOADED 0				;# True if program is loaded
# Directory containing configuration files
set CONFIG_DIR	"$::env(HOME)/.[string tolower ${::SHORTNAME}]"


## Tcl packages used by this prgram
 # Format: {
 # 	{pkg_name	pkg_verison}
 # 	...
 # }
set LIBRARIES_TO_LOAD {
	{BWidget	1.7}
	{Itcl		3.1}
	{Tcl		8.2}
	{md5		2.0}
	{crc16		1.1}
	{Tk		8.4}
	{img::png	1.3}
	{tdom		0.8}
}

# Handle CLI options
# -----------------------------
source "${::LIB_DIRNAME}/cli.tcl"

# SHOW WARNING MESSAGE
# -----------------------------

if {!$::CLI_OPTION(quiet)} {
	if {$::CLI_OPTION(nocolor)} {
		puts "WARNING:"
		puts "\tThis program is still under development."
		puts "\tThis program is distributed with ABSOLUTELY NO WARRANTY !"
		puts "\tPlease report bugs at http://mcu8051ide.sf.net"
		puts "\tAuthor: Martin Ošmera <martin.osmera@gmail.com>"
	} {
		puts "WARNING:"
		puts "\tThis program is still under development."
		puts "\tThis program is distributed with ABSOLUTELY NO WARRANTY !"
		puts "\tPlease report bugs at \033\[34;1mhttp://mcu8051ide.sf.net\033\[m"
		puts "\tAuthor: Martin Ošmera \033\[33;1m<martin.osmera@gmail.com>\033\[m"
	}
}

# PRE-INITIALIZATION
# -----------------------------
# Load Tk ToolKit
set T [lindex [time {
	package require img::png 1.3
	package require Tk 8.4
}] 0]
# Hide main window
wm withdraw .
update


# Load base config file
# -----------------------------
if {![file exists ${::CONFIG_DIR}]} {
	file mkdir ${::CONFIG_DIR}
	puts "\nCreating program configuration files in directory: \"[file normalize ${::CONFIG_DIR}]\""
	puts "Welcome in this IDE, [file tail [file normalize ~]] !"
}
if {[catch {
	set conf_file [open "${::CONFIG_DIR}/base.conf" r]
}]} {
	array set GLOBAL_CONFIG {
		splash		1
		tips		1
		language	en
	}
	
	if {[catch {
		set conf_file [open "${::CONFIG_DIR}/base.conf" w]
		puts -nonewline $conf_file {1 1 en}
		close $conf_file
	}]} {
		puts stderr "Unable to create base configuration file"
	}

} else {
	set data [read $conf_file]
	close $conf_file

	set GLOBAL_CONFIG(splash)	[lindex $data 0]
	set GLOBAL_CONFIG(tips)		[lindex $data 1]
	set GLOBAL_CONFIG(language)	[lindex $data 2]
	
	if {![string is boolean -strict ${GLOBAL_CONFIG(splash)}]} {
		set GLOBAL_CONFIG(splash) 1
	}
	if {![string is boolean -strict ${GLOBAL_CONFIG(tips)}]} {
		set GLOBAL_CONFIG(tips) 1
	}

	set tmp [glob -nocomplain -types f *.msg	\
		-directory "${LIB_DIRNAME}/../translations" -tails]
	set translations {en}
	foreach translation $tmp {
		lappend translations [file rootname $translation]
	}

	if {[lsearch $translations ${GLOBAL_CONFIG(language)}] == -1} {
		set GLOBAL_CONFIG(language) {en}
	}
}

# Load translation
# -----------------------------
if {${GLOBAL_CONFIG(language)} != {en}} {
	package require msgcat	1.3.4
	namespace import -force ::msgcat::*

	if {catch [{
		mclocale ${GLOBAL_CONFIG(language)}
		mcload "${LIB_DIRNAME}/../translations/${GLOBAL_CONFIG(language)}.msg"
	}] result} {
		puts stderr "Unable to load translation"
		puts stderr "\tFile: '${LIB_DIRNAME}/../translations/${GLOBAL_CONFIG(language)}.msg'"
		puts "\n"
		puts $result
	}
}

# CREATE SPLASH SCREEN
# -----------------------------
if {!$::CLI_OPTION(nosplash) && ${::GLOBAL_CONFIG(splash)}} {

	# Crete toplevel  window
	toplevel .splash -class {Splash creen}

	# Show image of splash creen
	place [label .splash.bg	\
		-image [	\
			image create photo -format png	\
				-file "${::LIB_DIRNAME}/../icons/other/splash.png"	\
			]	\
	] -x 0 -y 0 -width 400 -height 200

	# Show status bar
	place [label .splash.status		\
		-bg {#FFFFFF} -fg {#0000FF}	\
		-text "Initializing $APPNAME"	\
	] -x 200 -y 180 -anchor center

	# Set window parameters
	wm geometry .splash "=400x200+[expr {[winfo screenwidth .] / 2 - 200}]+[expr {[winfo screenheight .] / 2 - 100}]"
	wm overrideredirect .splash 1

	# Click on splash creen destroys it
	bind .splash <1> {wm withdraw .splash}

	# Done ..
	update
}

# BASIC FUNCTIONS
# -----------------------------

## Print content of $T in mili seconds ($T must contain value in [us])
 # @return void
proc time_in_msec {} {
	global T	;# Time in microseconds

	# Determinate number of miliseconds
	set msec [lindex $T 0]
	set msec [expr {$msec / 1000}]

	# print the message
	if {!$::CLI_OPTION(quiet)} {
		if {$::CLI_OPTION(nocolor)} {
			puts "\t... $msec ms"
		} {
			puts "\t... \033\[33m$msec ms\033\[m"
		}
	}
}

## This function should be called when some Tcl library fail to load
 # @parm String library - Name of failed library
 # @return void
proc libraryLoadFailed {library} {

	# Print error message
	if {$::CLI_OPTION(nocolor)} {
		puts stderr "\n\nERROR: Unable to load library $library"
	} {
		puts stderr "\n\n\033\[31mERROR:\033\[m Unable to load library \033\[32m$library\033\[m"
	}

	# Print tip
	puts "\nTip: try to run mcu8051ide with --check-libraries to see what's wrong."

	# Terminate the program
	exit 1
}

## Print some initialization message (splash screen and CLI)
 # @parm String message - text of the message
 # @return void
proc showInitMessage {message} {

	# Change content of splash screen status bar
	if {!${::CLI_OPTION(nosplash)} && ${::GLOBAL_CONFIG(splash)}} {
		.splash.status configure -text [string trim $message]
		update
	}

	# Print message to console output
	if {!${::CLI_OPTION(quiet)}} {
		if {${::CLI_OPTION(nocolor)}} {
			puts -nonewline $message
		} {
			puts -nonewline "\033\[37m$message\033\[m"
		}
		flush stdout
	}
}

## Set status bar tip for some widget
 # Usage:
 #	setStatusTip -widget $some_widget -text "some text"
 #
 # @return void
proc setStatusTip args {

	# Local variables
	set widgetIsSet	0		;# True if widget is set
	set textIsSet	0		;# True if text is set
	set argsLength	[llength $args]	;# Number of arguments
	set widget	{}		;# ID of widget specified by argument '-widget'
	set helpText	{}		;# Help text specified by argument '-text'

	# Iterate over given arguments and evaluate them
	for {set i 0} {$i < $argsLength} {incr i} {
		# Currently parsed argument
		set arg [lindex $args $i]
		# Decide what $arg means
		switch -- $arg {
			-widget {	;# ID of the widget

				# check if that widget wasn't already specified
				if {$widgetIsSet} {
					error "Widget has been already specified"
				}

				# Check if widget's ID follow the arument
				incr i
				if {$i >= $argsLength} {
					error "Expected widget name after -widget option"
				}

				# Set ID of the widget
				set widget [lindex $args $i]
				if {![winfo exists $widget]} {
					error "The specified widget does not exist"
				}

				# Widget is now set
				set widgetIsSet 1
			}

			-text {		;# The help text

				# Check if help text follow the argument
				incr i
				if {$i >= $argsLength} {
					error "Expected text after -text option"
				}

				# Set the help text
				set helpText [lindex $args $i]

				# Help text is now set
				set textIsSet 1
			}

			default {	;# Unrecognized opton -> invoke ERROR
				error "Invalid argument '$arg', possible options are -widget and -text"
			}
		}
	}

	# Ckeck if both aruments are properly specified
	if {!$widgetIsSet || !$textIsSet} {
		error "You must specify text and widget"
	}

	# Create binding
	bind $widget <Enter> "Sbar -freeze {$helpText}"
	bind $widget <Leave> "Sbar {}"
}



# INITIALIZATION
# -----------------------------

# Show "first line message"
if {!$CLI_OPTION(quiet)} {
	if {$CLI_OPTION(nocolor)} {
		puts "\nInitializing MCU 8051 IDE ${VERSION}"
	} {
		puts "\nInitializing \033\[1mMCU 8051 IDE \033\[32m${VERSION}\033\[m"
	}
}

## Load libraries
showInitMessage "\tLoading libraries"
incr T [lindex [time {
	# Iterate over list of libraries and lod each of them
	foreach library $::LIBRARIES_TO_LOAD {
		# Loading successful
		if {[catch {package require [lindex $library 0] [lindex $library 1]}]} {
			libraryLoadFailed [lindex $library 0]
		# Loading failed
		} {
			if {!$::CLI_OPTION(nosplash)} update
		}
	}
	# Import NS for managing OOP in Tcl
	namespace import -force ::itcl::*
}] 0]
# Look for some external programs
foreach program {
		urxvt	vim	emacs	kwrite	gedit	nano	dav 
		le	sdcc	indent	doxygen	asl	asem	doxywizard
	} {
		if {[auto_execok $program] == {}} {
			set ::PROGRAM_AVALIABLE($program) 0
		} {
			set ::PROGRAM_AVALIABLE($program) 1
		}
}
time_in_msec	;# Print time info

## Load program sources
showInitMessage "\tLoading program sources"
set T [time {
	source "${::LIB_DIRNAME}/innerwindow.tcl"	;#
	source "${::LIB_DIRNAME}/errorhandler.tcl"	;# Background error handler
	source "${::LIB_DIRNAME}/settings.tcl"		;# Settings management
	source "${::LIB_DIRNAME}/project.tcl"		;# Project management
	source "${::LIB_DIRNAME}/fsd.tcl"		;# File selection dialog
	source "${::LIB_DIRNAME}/external_compiler.tcl"	;# External compiler interface
	source "${::LIB_DIRNAME}/X.tcl"			;# GUI <==> Implementation Interface
	source "${::LIB_DIRNAME}/configdialogs.tcl"	;# Configuration dialogs
	source "${::LIB_DIRNAME}/editor.tcl"		;# Source code editor
	source "${::LIB_DIRNAME}/Math.tcl"		;# Special mathematical operations
	source "${::LIB_DIRNAME}/compiler.tcl"		;# 8051 Assemly language compiler
	source "${::LIB_DIRNAME}/tips.tcl"		;# Tips on startup
	source "${::LIB_DIRNAME}/hexeditor.tcl"		;# Hexadecimal editor
	source "${::LIB_DIRNAME}/hexeditdlg.tcl"	;# Hexadecimal editor dialog
	source "${::LIB_DIRNAME}/enviroment.tcl"	;# Main window "trappings" (menu and such)
	source "${::LIB_DIRNAME}/filelist.tcl"		;# Left and middle panel
	source "${::LIB_DIRNAME}/bottomnotebook.tcl"	;# Bottom panel
	source "${::LIB_DIRNAME}/maintab.tcl"		;# Central widget
	source "${::LIB_DIRNAME}/ihextools.tcl"		;# Tools for manipulating IHEX8
	source "${::LIB_DIRNAME}/symbol_viewer.tcl"	;# Assembly symbols viewer
	source "${::LIB_DIRNAME}/eightsegment.tcl"	;# 8-segment LED display editor
	source "${::LIB_DIRNAME}/asciichart.tcl"	;# ASCII chart
	source "${::LIB_DIRNAME}/notes.tcl"		;# Scribble notes
}]
time_in_msec	;# Print time info

# CHECK FOR VALIDITY OF THE MCU DATABASE
if {${::X::avaliable_processors} == {}} {
	tk_messageBox		\
		-icon error	\
		-type ok	\
		-title "FATAL ERROR"	\
		-message "MCUs database file is currupted,\nthis program cannot run without it.\nPlease reinstall MCU 8051 IDE."
	exit 1
}

# Load global configuration
loadApplicationConfiguration

# Initialize GUI enviroment
iconbar_redraw		;# Main toolbar
mainmenu_redraw		;# Main menu
shortcuts_reevaluate	;# Key shortcuts

## Remove splash screen
destroy .splash
if {$CLI_OPTION(minimalized)} {
	wm state . iconic
} {
	wm deiconify .
}

## Open the last session
# Print message
if {!$CLI_OPTION(quiet)} {
	if {$CLI_OPTION(nocolor)} {
		puts -nonewline "\tOpening last session"
	} {
		puts -nonewline "\t\033\[37mOpening last session\033\[m"
	}
}
flush stdout
# Evaluate new geometry of the main window
evaluate_new_window_geometry
# Open projects of last session
set T [time {
	foreach project_file $::CONFIG(OPENED_PROJECTS) {
		if {![Project::open_project_file $project_file]} {
			tk_messageBox			\
				-title "File not found"	\
				-icon warning		\
				-type ok		\
				-message "Unable to open project file:\n\"$project_file\""
		}
	}
}]
# Focus on active editor
if {${::X::actualProject} != {}} {
	focus [${::X::actualProject} editor_procedure {} cget -editor]
}
time_in_msec	;# Print time info

# Create binding for panes management
bind . <Configure> {X::redraw_panes}
# Print final message
if {!$CLI_OPTION(quiet)} {
	puts "$APPNAME is now operational\n"
}

# Program is now operational
set APPLICATION_LOADED 1
set X::critical_procedure_in_progress 0
update idle
X::redraw_panes